# -*- mode: python; coding: utf-8 -*-
# vim:smartindent cinwords=if,elif,else,for,while,try,except,finally,def,class:ts=4:sts=4:sta:et:ai:shiftwidth=4
#
# Copyright © 2003,2004 Colin Walters <walters@verbum.org>
# Copyright © 2004, 2006 Canonical Ltd. 
#	Author: Robert Collins <robertc@robertcollins.net>
# Copyright © 2003, 2005 Walter Landry

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import errno
import os
import sys


class FailedToLock(Exception):
    """The lock could not be acquired."""


class LockFile(object):
    """I represent a lock that is made on the file system, to prevent concurrent execution of this code"""

    def __init__(self, filename, logger, no_act, cron_mode):
        self.filename = filename
        self.locked = False
        self.logger = logger
        self.no_act = no_act
        self.cron_mode = cron_mode

    def acquire(self):
        if self.no_act:
            return
        self.logger.info('creating lockfile')
        try:
            fd = os.open(self.filename, os.O_CREAT | os.O_EXCL | os.O_WRONLY)
            lockfile = os.fdopen(fd, 'wt')
            lockfile.write("%s\n"% os.getpid())
            lockfile.close()
            self.locked=True
        except OSError, e:
            if e.errno != errno.EEXIST:
                raise
            old_lock =open(self.filename, 'rt')
            old_pid = int(old_lock.read())
            old_lock.close()
            try:
                # stale processes raise ESRCH, other errors should never 
                # happen, and existing processes return None
                os.kill(old_pid, 0)
            except OSError, e:
                if e.errno != errno.ESRCH:
                    raise
                # stale process
                os.unlink(self.filename)
                return self.acquire()
            if self.cron_mode:
                self.logger.info("lockfile %s already exists, exiting", self.filename)
                sys.exit(0)
            else:
                self.logger.error("Couldn't create lockfile: %s", self.filename)
                raise FailedToLock

    def release(self):
        if not self.locked:
            return
        if self.no_act:
            return
        self.logger.debug('Removing lock file: %s', self.filename)
        os.unlink(self.filename)
        self.locked = False

