# -*- mode: python; coding: utf-8 -*-
# vim:smartindent cinwords=if,elif,else,for,while,try,except,finally,def,class:ts=4:sts=4:sta:et:ai:shiftwidth=4
#
# Copyright © 2008 Canonical Ltd.
#	Author: Tim Penhey <tim@canonical.com>

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

class Queue(object):
    """The Queue contains scripts for the Patch Queue Manager to do.

    The queue contains instances of the type `Script`.
    """

    def is_processing_requests(self):
        """Is the queue currently processing requests?"""
        raise NotImplementedError()

    def next_script(self):
        """The next script to process for workers.

        Queues do not expose a way to remove scripts - scripts do that as 
        part of their own handling.

        :return: A `Script` or None if there is nothing to do or the queue
            is paused.
        """
        raise NotImplementedError()

    def items(self):
        """The scripts currently in the queue.

        This method is used primarily by the UI to show upcoming work.
        """
        raise NotImplementedError()
