# -*- mode: python; coding: utf-8 -*-
# vim:smartindent cinwords=if,elif,else,for,while,try,except,finally,def,class:ts=4:sts=4:sta:et:ai:shiftwidth=4
#
# Copyright © 2004, 2008 Canonical Ltd.
#	Author: Robert Collins <robertc@robertcollins.net>
#	Author: Tim Penhey <tim@canonical.com>

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import os
import re
import stat

from pqm.queue import Queue
from pqm.script import EmailScript


def find_patches(queuedir, logger, branch_spec_handler, configp, options):
    patches=[]
    patches_re=re.compile('^patch\.\d+$')
    for f in os.listdir(queuedir):
        if patches_re.match(f):
            fname=os.path.join(queuedir, f)
            submission_time = os.stat(fname)[stat.ST_MTIME]
            script = EmailScript(fname, logger, options.verify_sigs,
                                 submission_time, branch_spec_handler,
                                 configp, options.keyring)
            patches.append((script, f))
    def sortpatches(a, b):
        return cmp(a[1], b[1])
    patches.sort(sortpatches)
    return [patch[0] for patch in patches]


class EmailQueue(Queue):
    """The email queue gets the scripts from the patch files in the queuedir.

    """

    def __init__(self, queuedir, logger, branch_spec_handler, configp,
                 options):
        self.queuedir = queuedir
        self.configp = configp
        self.scripts = find_patches(
            queuedir, logger, branch_spec_handler, configp, options)

    def is_processing_requests(self):
        """Is the queue currently processing requests?"""
        return not os.path.isfile("%s/stop.patch" % self.queuedir)

    def next_script(self):
        """The next script to process.

        :return: A `Script` or None if there is nothing to do or the queue
            is paused.
        """
        if self.is_processing_requests() and len(self.scripts) > 0:
            return self.scripts.pop(0)
        else:
            return None

    def items(self):
        """The scripts currently in the queue.

        This method is used primarily by the UI to show upcoming work.
        """
        return self.scripts
