# -*- mode: python; coding: utf-8 -*-
# vim:smartindent cinwords=if,elif,else,for,while,try,except,finally,def,class:ts=4:sts=4:sta:et:ai:shiftwidth=4
#
# Copyright © 2010 Canonical Ltd.
#	Author: Robert Collins <robert@canonical.com>

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

__all__ = ['ListQueue']

from pqm.queue import Queue

class ListQueue(Queue):
    """A queue that combines other queues.
    
    :ivar queues: The queues being combined. Items are returned from the 
        first queue until its empty, then the second, and so on.
    """

    def __init__(self, queues):
        """Create a combined queue examining queues."""
        self.queues = queues

    def is_processing_requests(self):
        # True if any of the queues is processing requests.
        return reduce(lambda val, queue: val or queue.is_processing_requests(),
            self.queues, False)

    def next_script(self):
        for queue in self.queues:
            result = queue.next_script()
            if result is not None:
                return result
        return None

    def items(self):
        result = []
        for queue in self.queues:
            result.extend(list(queue.items()))
        return result
