# -*- mode: python; coding: utf-8 -*-
# vim:smartindent cinwords=if,elif,else,for,while,try,except,finally,def,class:ts=4:sts=4:sta:et:ai:shiftwidth=4
#
# Copyright © 2006 Canonical Ltd.
#	Author: Robert Collins <robertc@robertcollins.net>

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import logging
import os

from twisted.trial import unittest

import pqm.lockfile as lockfile


class TestStaleLock(unittest.TestCase):

    def tearDown(self):
        try:
            os.unlink(self.lockname)
        except OSError:
            pass

    def testMakeStaleLock(self):
        self.lockname = "shouldbestale"
        pid = self.makeStaleLock()
        self.failUnless(os.path.isfile(self.lockname))
        self.assertEqual("%s\n" % pid, open(self.lockname, 'rt').read())

    def testNonStaleLocksDontAcquire(self):
        self.lockname = "activelock"
        lock = self.makeLock()
        lock.acquire()
        secondlock = self.makeLock()
        self.assertRaises(lockfile.FailedToLock, secondlock.acquire)
        lock.release()

    def testStaleLocksAllowAcquiring(self):
        self.lockname = 'stalelock'
        pid = self.makeStaleLock()
        lock = self.makeLock()
        lock.acquire()
        self.assertEqual("%s\n" % os.getpid(), open(self.lockname, 'rt').read())
        lock.release()

    def makeStaleLock(self):
        """Make a stale lock at self.lockname."""
        pid = os.fork()
        if pid == 0:
            lock = self.makeLock()
            lock.acquire()
            os._exit(0)
        else:
            waitpid, waitresult = os.waitpid(pid, 0)
            self.failUnless(os.WIFEXITED(waitresult))
            self.assertEqual(0, os.WEXITSTATUS(waitresult))
            return pid

    def makeLock(self):
        return lockfile.LockFile(self.lockname, logging, no_act=False,
                                 cron_mode=False)
