import os

from twisted.trial import unittest

import pqm
from pqm.resultsummary import TestStatusSummary

from subunit import PROGRESS_SET


class TestTestStatusSummary(unittest.TestCase):
    """Smoke tests of TestStatusSummary."""

    def setUp(self):
        super(TestTestStatusSummary, self).setUp()
        if os.path.exists('statusfile'):
            os.unlink('statusfile')
        status = pqm.StatusFile('statusfile')
        self.summary = TestStatusSummary(status)

    def status_lines(self):
        return file('statusfile').readlines()

    def status_line(self):
        return self.status_lines()[-1]

    def test_status_height_set(self):
        self.assertEqual(1, self.summary.status.status_area)

    def test_initial_message(self):
        self.assertEqual("No subunit data.\n", self.status_line())

    def test_startTestRun(self):
        self.summary.startTestRun()
        self.assertEqual("Running [ 0% ] Current test: None\n",
            self.status_line())

    def test_stopTestRun(self):
        self.summary.stopTestRun()
        self.assertEqual("Finished [ 0% ]\n", self.status_line())

    def test_startTest(self):
        self.summary.startTestRun()
        self.summary.startTest(self)
        self.assertEqual("Running [ 0% 1 test(s) ] Current test: pqm.tests.test_resultsummary.TestTestStatusSummary.test_startTest\n",
            self.status_line())
        self.summary.stopTest(self)
        self.summary.stopTestRun()

    def test_stopTest(self):
        self.summary.startTestRun()
        self.summary.startTest(self)
        self.summary.stopTest(self)
        self.assertEqual("Running [ 0% 1 test(s) ] Current test: None\n",
            self.status_line())
        self.summary.stopTestRun()

    def test_progress(self):
        self.summary.startTestRun()
        self.summary.progress(10, PROGRESS_SET)
        self.summary.startTest(self)
        self.summary.stopTest(self)
        self.assertEqual("Running [ 10% 1 test(s) ] Current test: None\n",
            self.status_line())
        self.summary.stopTestRun()

    def test_fail(self):
        self.summary.startTestRun()
        self.summary.progress(10, PROGRESS_SET)
        self.summary.startTest(self)
        try:
            raise AssertionError("foo bar")
        except AssertionError, e:
            pass
        self.summary.addFailure(self, self._exc_info())
        self.summary.stopTest(self)
        status_lines = self.status_lines()
        status_lines[7] = 'stubbed out\n'
        self.assertEqual(['\n',
            '\n',
            '\n',
            '\n',
            '\n',
            'Failure pqm.tests.test_resultsummary.TestTestStatusSummary.test_fail\n',
            'Traceback (most recent call last):\n',
            'stubbed out\n',
            '    raise AssertionError("foo bar")\n',
            'AssertionError: foo bar\n',
            '\n',
            "Running [ 10% 1 test(s) 1 failure(s) ] Current test: None\n"],
            status_lines)
        self.summary.stopTestRun()
